; --------------------------------------------------------------------------
; AVR Thermometer
; (c) 2003 Andreas Schroeder <andreas@a-netz.de>
;
; Module: 16 Bit Delay
; --------------------------------------------------------------------------

.DEF RTim = R25

.EQU TIMER_BIT_RUNNING = 0

; --------------------------------------------------------------------------
; Timer interval definition
; divider 3 (= 1/64) at 4 MHz means base interval of 16 us - max is 1sec
; --------------------------------------------------------------------------
.EQU TIMER1_DIVIDER = 0x03

; --------------------------------------------------------------------------
; Initialization: enable the timer 1 compare interrupt
; interrupts have to be enabled with sei
; --------------------------------------------------------------------------
.MACRO timer_init
	; enable timer 1 compare interrupt
	in		RTim, TIMSK
	ori		RTim, (1 << OCIE1A)
	out		TIMSK, RTim

	; reset timer 1 counter register
 	ldi		RTim, 0
 	out		TCNT1H, RTim
 	out		TCNT1L, RTim

 	; disable the timer at startup
 	out		TCCR1B, RTim
.ENDMACRO


; --------------------------------------------------------------------------
; Proc: Start the timer (either 8 or 16 bit)
;
; Params: RP0 - low byte, RP1 - high byte of delay value
; --------------------------------------------------------------------------
.CSEG

; this is used for 8-bit values, hi byte is set to 0
timer_start8:
	ldi		RP1, 0

; this is the real 16 bit timer function
timer_start16:
	out		OCR1AH, RP1
	out		OCR1AL, RP0

 	; enable timer 1 and set the clear on compare match flag
 	ldi		RTim, TIMER1_DIVIDER | 0x08
 	out		TCCR1B, RTim

 	; set bit 0 in timer register to indicate that timer
 	; is running now
 	sbr		RTim, TIMER_BIT_RUNNING

 	; sleep and wait for the timer 1 compare interrupt
__timerloop:
 	sleep
	sbrc	RTim, TIMER_BIT_RUNNING
 	rjmp	__timerloop

	ret

; --------------------------------------------------------------------------
; Proc: Timer 1 Compare Interrupt
; --------------------------------------------------------------------------
timer1compare:
 	; disable the timer
 	ldi		RTim, 0
 	out		TCCR1B, RTim

 	cbr		RTim, TIMER_BIT_RUNNING

 	reti
